import boto3
import base64
import json

from django_configuration_management.validation_utils import (
    validate_key_name,
)


def get_secret(secret_name):
    client = boto3.client("secretsmanager")

    get_secret_value_response = client.get_secret_value(SecretId=secret_name)

    if "SecretString" in get_secret_value_response:
        secret = get_secret_value_response["SecretString"]
    else:
        secret = base64.b64decode(get_secret_value_response["SecretBinary"])

    return secret


def parse_secret(secret_name: str, secret_keys: list) -> dict:
    secret_object = json.loads(get_secret(secret_name))
    secrets = dict()
    for secret_key in secret_keys:
        validate_key_name(secret_key)
        assert (
            secret_key in secret_object
        ), f"{secret_name} doesn't contain the key {secret_key}"
        secrets[secret_key] = secret_object[secret_key]
    return secrets


def pull_aws_config_data(data: dict):
    pulled = dict()
    for key, meta in data.items():
        secret_keys = meta["secret_keys"]

        assert (
            type(secret_keys) == list and len(secret_keys) > 0
        ), f"{key} must have a list of secret_keys"

        secret_object = parse_secret(key, secret_keys)
        pulled = {**pulled, **secret_object}
    return pulled


from django.conf import settings

from django_configuration_management.aws_utils import pull_aws_config_data
from django_configuration_management.utils import (
    load_env,
    normalize_config_data,
)
from django_configuration_management.yml_utils import yml_to_dict


def get_config(environment: str, dotenv_required=True):
    try:
        load_env(environment)
    except AssertionError as error:
        if dotenv_required:
            raise error

    local_secrets, aws_secrets = yml_to_dict(environment)

    normalized_local_secrets = normalize_config_data(local_secrets)
    pulled_aws_secrets = pull_aws_config_data(aws_secrets)

    return {**normalized_local_secrets, **pulled_aws_secrets}


def inject_config(environment: str, settings_module: settings, dotenv_required=True):
    config = get_config(environment, dotenv_required)

    for key, value in config.items():
        setattr(settings_module, key, value)



import click

from django_configuration_management.aws_utils import pull_aws_config_data
from django_configuration_management.secrets import (
    decrypt_value,
    encrypt_value,
    generate_fernet_key,
)
from django_configuration_management.utils import gather_user_input, load_env
from django_configuration_management.yml_utils import dict_to_yml, yml_to_dict


@click.command("upsert_secret")
@click.option("-e", "--environment", default="development", help="Your environment")
def upsert_secret(environment):
    load_env(environment)

    data = yml_to_dict(environment, skip_required_checks=True)
    key_name, key_value = gather_user_input()

    data[key_name] = {"value": key_value, "secret": True}

    dict_to_yml(data, environment)


@click.command("reveal_secrets")
@click.option("-e", "--environment", default="development", help="Your environment")
def reveal_secrets(environment):
    load_env(environment)

    local_secrets, aws_secrets = yml_to_dict(environment, skip_required_checks=True)

    for key, meta in local_secrets.items():
        # Skip non-secret values
        if type(meta) != dict:
            continue

        value = meta["value"]
        secret = decrypt_value(value)
        print(f"{key}={secret}")

    for key, secret in pull_aws_config_data(aws_secrets).items():
        print(f"{key}={secret}")


@click.command("generate_key")
def generate_key():
    key = generate_fernet_key()

    print(f"Your key is: \n\n{key}\n")
    print(
        "Please store this in whichever .env-[environment] file you generated it for under the variable ENC_KEY"
    )


@click.command("reencrypt")
@click.option("-e", "--environment", default="development", help="Your environment")
@click.option(
    "-k",
    "--new-key",
    help="The new key with which you'd like to re-encrypt your secrets",
)
def reencrypt(environment, new_key):
    load_env(environment)

    if not new_key:
        new_key = generate_fernet_key()

    data = yml_to_dict(environment, skip_required_checks=True)

    for key, meta in data.items():
        # Skip non-secret values
        if type(meta) != dict:
            continue

        value = meta["value"]
        decrypted_value = decrypt_value(value)
        meta["value"] = encrypt_value(decrypted_value, enc_key=new_key)

        data[key] = meta

    dict_to_yml(data, environment)

    print(f"Your new key is: \n\n{new_key}\n")
    print(
        f"config-{environment}.yaml has been re-encrypted. Please be sure to update your .env-{environment} with the new key you used!"
    )



import json
import re

from pathlib import Path


def validate_key_name(key_name: str):
    assert re.search(
        "^[A-Z0-9]+(?:_[A-Z0-9]+)*$", key_name
    ), f"Invalid key name {key_name}. Keys must consist only of uppercase letters and underscore"


def read_required_vars_file():
    required_vars = Path(".") / "config-required.json"

    try:
        with open(required_vars, "r") as file:
            required_vars = json.load(file)
    except FileNotFoundError:
        return

    return required_vars